/*
 * @(#)src/classes/sov/javax/swing/plaf/basic/BasicRootPaneUI.java, swing, as1411, 20040217 1.9.2.1
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 * IBM SDK, Java(tm) 2 Technology Edition, v1.4.1
 * (C) Copyright IBM Corp. 2002, 2003. All Rights Reserved
 * ===========================================================================
 */

/*
 * ===========================================================================
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * ===========================================================================
 */


/* 
 *
 * Change activity:
 *
 * Reason Date   Origin   Description
 * ------ ------ -------- --------------------------------------------------- 
 * 57093  130103 chenerys Add Shift+F10 key binding to post contextual menus
 *
 * ===========================================================================
 */

package javax.swing.plaf.basic;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;/*ibm@57093*/
import java.awt.event.KeyEvent;/*ibm@57093*/
import java.awt.event.MouseEvent;/*ibm@57093*/
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.*;
import javax.swing.plaf.*;

/**
 * Basic implementation of RootPaneUI, there is one shared between all
 * JRootPane instances.
 *
 * @version 1.8 04/18/02
 * @author Scott Violet
 */
public class BasicRootPaneUI extends RootPaneUI implements
                  PropertyChangeListener {
    private static RootPaneUI rootPaneUI = new BasicRootPaneUI();

    public static ComponentUI createUI(JComponent c) {
        return rootPaneUI;
    }

    public void installUI(JComponent c) { 
        installDefaults((JRootPane)c);
        installComponents((JRootPane)c);
        installListeners((JRootPane)c);
        installKeyboardActions((JRootPane)c);
    }

    
    public void uninstallUI(JComponent c) { 
        uninstallDefaults((JRootPane)c);
        uninstallComponents((JRootPane)c);
        uninstallListeners((JRootPane)c);
        uninstallKeyboardActions((JRootPane)c);
    }

    protected void installDefaults(JRootPane c){
    }

    protected void installComponents(JRootPane root) {
    }

    protected void installListeners(JRootPane root) {
	root.addPropertyChangeListener(this);
    }

    protected void installKeyboardActions(JRootPane root) {
	InputMap km = getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW, root);
	SwingUtilities.replaceUIInputMap(root, JComponent.WHEN_IN_FOCUSED_WINDOW,
				       km);
	ActionMap am = getActionMap(root);
	SwingUtilities.replaceUIActionMap(root, am);
	updateDefaultButtonBindings(root);
    }

    protected void uninstallDefaults(JRootPane root) {
    }

    protected void uninstallComponents(JRootPane root) {
    }

    protected void uninstallListeners(JRootPane root) {
	root.removePropertyChangeListener(this);
    }

    protected void uninstallKeyboardActions(JRootPane root) {
	SwingUtilities.replaceUIInputMap(root, JComponent.
				       WHEN_IN_FOCUSED_WINDOW, null);
	SwingUtilities.replaceUIActionMap(root, null);
    }

    InputMap getInputMap(int condition, JComponent c) {
	if (condition == JComponent.WHEN_IN_FOCUSED_WINDOW) {
	    return createInputMap(condition, c);
	}
	return null;
    }

    ActionMap getActionMap(JComponent c) {
	return createActionMap(c);
    }

    ComponentInputMap createInputMap(int condition, JComponent c) {
        /*ibm@57093 start*/
        ComponentInputMap im = new RootPaneInputMap(c);
        Object[] bindings = (Object[])UIManager.get("RootPane.windowBindings");
        if (bindings != null) {
            LookAndFeel.loadKeyBindings(im, bindings);
        }
        return im;
        /*ibm@57093 end*/
    }

    ActionMap createActionMap(JComponent c) {
	ActionMap map = new ActionMapUIResource();

	map.put("press", new DefaultAction((JRootPane)c, true));
	map.put("release", new DefaultAction((JRootPane)c, false));
        map.put("postpopup", new PostPopupAction());       /*ibm@57093*/
	return map;
    }

    /**
     * Invoked when the default button property has changed. This reloads
     * the bindings from the defaults table with name
     * <code>RootPane.defaultButtonWindowKeyBindings</code>.
     */
    void updateDefaultButtonBindings(JRootPane root) {
	InputMap km = SwingUtilities.getUIInputMap(root, JComponent.
					       WHEN_IN_FOCUSED_WINDOW);
	while (km != null && !(km instanceof RootPaneInputMap)) {
	    km = km.getParent();
	}
	if (km != null) {
            clearDefaultButtonBindings(km); /*ibm@57093*/
	    if (root.getDefaultButton() != null) {
		Object[] bindings = (Object[])UIManager.get
		          ("RootPane.defaultButtonWindowKeyBindings");
		if (bindings != null) {
		    LookAndFeel.loadKeyBindings(km, bindings);
                    oldDefaultButtonBindings=bindings; /*ibm@57093*/
		}
	    }
	}
    }

    /*ibm@57093 start*/
    private Object[] oldDefaultButtonBindings=null;    

    private void clearDefaultButtonBindings(InputMap im){
        if (oldDefaultButtonBindings != null) {
            for (int counter = 0, maxCounter = oldDefaultButtonBindings.length;
                 counter < maxCounter; counter++) {
                Object keyStrokeO = oldDefaultButtonBindings[counter++];
                KeyStroke ks = (keyStrokeO instanceof KeyStroke) ?
                    (KeyStroke)keyStrokeO :
                    KeyStroke.getKeyStroke((String)keyStrokeO);
                im.remove(ks);
            }
            oldDefaultButtonBindings=null;
        }
    }
    /*ibm@57093 end*/

    /**
     * Invoked when a property changes on the root pane. If the event
     * indicates the <code>defaultButton</code> has changed, this will
     * reinstall the keyboard actions.
     */
    public void propertyChange(PropertyChangeEvent e) {
	if(e.getPropertyName().equals("defaultButton")) {
	    JRootPane rootpane = (JRootPane)e.getSource();
	    updateDefaultButtonBindings(rootpane);
	    if (rootpane.getClientProperty("temporaryDefaultButton") == null) {
		rootpane.putClientProperty("initialDefaultButton", e.getNewValue());
	    }
	}
    }


    // This was transplanted from JRootPane.
    static class DefaultAction extends AbstractAction {
        JRootPane root;
        boolean press;
        DefaultAction(JRootPane root, boolean press) {
            this.root = root;
            this.press = press;
        }
        public void actionPerformed(ActionEvent e) {
	    JButton owner = root.getDefaultButton();
            if (owner != null && SwingUtilities.getRootPane(owner) == root) {
                if (press) {
                    owner.doClick(20);
                }
            }
        }
        public boolean isEnabled() {
	    JButton owner = root.getDefaultButton();
            return (owner != null && owner.getModel().isEnabled());
        }
    }

    /*ibm@57093 start*/  
    private static class PostPopupAction extends AbstractAction {
	public void actionPerformed(ActionEvent e) {
            java.awt.Component focusOwner=
                java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
            if(focusOwner==null){
                focusOwner=
                    java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusedWindow();
            }
            if(focusOwner!=null){
                boolean menuBarActive=false;
                if(focusOwner instanceof JRootPane){
                    MenuElement[] path=MenuSelectionManager.defaultManager().getSelectedPath();
                    if(path[0]==((JRootPane)focusOwner).getJMenuBar()){
                        menuBarActive=true;
                    }
                }
                if(!menuBarActive){
                    java.awt.Dimension s=focusOwner.getSize();
                    MouseEvent mpe=new MouseEvent(focusOwner,MouseEvent.MOUSE_PRESSED,
                                                  System.currentTimeMillis(),0,s.width/2,s.height/2,1,true);
                    focusOwner.dispatchEvent(mpe);
                    MouseEvent mre=new MouseEvent(focusOwner,MouseEvent.MOUSE_RELEASED,
                                                  System.currentTimeMillis(),0,s.width/2,s.height/2,1,true);
                    focusOwner.dispatchEvent(mre);
                    MouseEvent mce=new MouseEvent(focusOwner,MouseEvent.MOUSE_CLICKED,
                                                  System.currentTimeMillis(),0,s.width/2,s.height/2,1,true);
                    focusOwner.dispatchEvent(mce);
                }
            }
        }
    }
    /*ibm@57093 end*/

    private static class RootPaneInputMap extends ComponentInputMapUIResource {
	public RootPaneInputMap(JComponent c) {
	    super(c);
	}
    }
}
